from firedrake import *
from firedrake.tsfc_interface import compile_form
import numpy

msh = UnitTriangleMesh()

flops = []
for el, deg in zip(["Lagrange"]*4, [2, 3, 4, 5]):
    V = FunctionSpace(msh, el, deg)

    n = FacetNormal(msh)
    h = CellSize(msh)
    h_avg = (h('+') + h('-'))/2.0

    mu = Constant(0.225)
    alpha = Constant(8.0)
    u = TrialFunction(V)
    v = TestFunction(V)

    # Define bilinear form
    a = (inner(div(grad(u)), div(grad(v)))*dx -
         inner(avg(div(grad(u))), jump(grad(v), n))*dS -
         inner(jump(grad(u), n), avg(div(grad(v))))*dS +
         alpha/h_avg*inner(jump(grad(u), n), jump(grad(v), n))*dS)

    kernels = compile_form(a, "biharmonic_dg_{}_{}".format(el, deg))
    cell_flops = sum([k.kinfo.kernel.num_flops for k in kernels
                      if (k.kinfo.integral_type == 'cell')])
    edge_flops = sum([k.kinfo.kernel.num_flops for k in kernels
                      if (k.kinfo.integral_type == 'interior_facet')])
    flops.append(["{$P^%d$}" % deg, cell_flops, 1.5*edge_flops])

for el, deg in zip(["Morley", "Bell", "Argyris"], [2, 5, 5]):
    V = FunctionSpace(msh, el, deg)

    n = FacetNormal(msh)
    h = CellSize(msh)
    beta = Constant(20)

    mu = Constant(0.225)

    u = TrialFunction(V)
    v = TestFunction(V)

    a = (inner(u.dx(0).dx(0), v.dx(0).dx(0)) +
         inner(u.dx(1).dx(1), v.dx(1).dx(1)) +
         mu * inner(u.dx(1).dx(1), v.dx(0).dx(0)) +
         mu * inner(u.dx(0).dx(0), v.dx(1).dx(1)) +
         2*(1-mu)*inner(u.dx(0).dx(1), v.dx(0).dx(1)))*dx

    kernel, = compile_form(a, "biharmonic_{}_{}".format(el, deg))

    flops.append([el, kernel.kinfo.kernel.num_flops, 0])

flops = numpy.asarray(flops, dtype=object)

with open("data/biharmonicflops.csv", "w") as f:
    numpy.savetxt(f, flops, fmt=["%s", "%e", "%e"], delimiter=",",
                  header="Name, Cell Flops, Edge Flops", comments="")
