# plots sinusoidally perturbed unitsquare mesh
from firedrake import *
import os

if not os.path.exists("pictures"):
    os.makedirs("pictures")
elif not os.path.isdir("pictures"):
    raise RuntimeError("Cannot create output directory, file of given name exists")

N = 8
msh_base = UnitSquareMesh(N, N)
V = FunctionSpace(msh_base, msh_base.coordinates.ufl_element())
eps = Constant(3.0 / 2**(N-1))

x, y = SpatialCoordinate(msh_base)
new = Function(V).interpolate(as_vector([x + eps*sin(2*pi*x)*sin(2*pi*y),
                                         y - eps*sin(2*pi*x)*sin(2*pi*y)]))
msh_base = Mesh(new)


coords = msh_base.coordinates.dat.data

ltg = msh_base.coordinates.cell_node_map().values

with open("pictures/tikzmesh.tex", "w") as f:
    f.write("\\begin{tikzpicture}[scale=3.0]\n")
    for c in range(ltg.shape[0]):
        xs = tuple(coords[ltg[c, :]].real.reshape((-1,)))
        f.write("\\draw (%e, %e) -- (%e, %e) -- (%e, %e) -- cycle;\n" % xs)
    f.write("\\end{tikzpicture}\n")
